from pgu import engine, gui, timer
import pygame
from pygame.locals import *
import webbrowser
import os
import urllib


import game_config

# scenes
from slideshowviewer import PlaySlideShow
from credits import ViewCredits
from editor import LevelEditor
import levels
import textscroller

class LevelFinished(engine.State):
    pass
class SelectLevel(engine.State):
    pass
class ViewHiscores(engine.State):
    pass


class ExitScene(engine.State):
    pass


class MainMenu(engine.State):
    background = os.path.join("data", "power_plant.jpg")
    sound_file = os.path.join("data", "menu.ogg")
    def init(self):
        self.next_state = None
        self.app = gui.App(theme=gui.Theme(["data/themes/default"]))
        self.image = pygame.transform.scale(pygame.image.load(self.background), 
                              (self.game.parameters.screenWidth, self.game.parameters.screenHeight))
        self.sound = pygame.mixer.Sound( self.sound_file )
        self.gui_init()
        self.sound.play(-1)
       
    def event(self, evt):
        return self.app.event(evt)
        
    
        
    def loop(self):
        if self.next_state:
            self.sound.stop()
            return self.next_state
        
    def paint(self, screen):
        screen.blit(self.image,(0,0))
        self.app.paint(screen)
        
        pygame.display.flip()
        
    def update(self, screen):
        self.paint(screen)
         
    def gui_init(self):
        space = 10
        table = gui.Table()

        table.tr()
        new_game = gui.Button("Play")
        new_game.connect(gui.CLICK, self.onNewGame, None)        
        table.td( new_game )
        table.tr(); table.td( gui.Spacer(0, space) )
        
        #table.tr()
        #hiscores = gui.Button("View Hiscores")
        #hiscores.connect(gui.CLICK, self.onHiscores, None)
        #table.td( hiscores )
        
        table.tr(); table.td( gui.Spacer(0, space) )
        
        table.tr()
        credits = gui.Button("Credits")
        credits.connect(gui.CLICK, self.onCredits, None)
        table.td( credits )

        table.tr(); table.td( gui.Spacer(0, space) )
        
        table.tr()
        exit = gui.Button("Exit")
        exit.connect(gui.CLICK, self.onExit, None)
        
        table.td( exit )
        
        cont = gui.Container()
        cont.add(table, 0, 0)
        self.app.init(cont)
        
    def onHiscores(self, *args):
        self.next_state = ViewHiscores(self.game)
    
    def onCredits(self, *args):
        self.next_state = ViewCredits(self.game, MainMenu)
        
    def onExit(self, *args):
        self.next_state = engine.Quit(self.game)#ExitScene
        
    def onNewGame(self, *args):
        self.next_state = SelectLevel(self.game, MainMenu)


LevelList = [
    ('Level 1',[levels.LevelOne]), 
    ('Level 2',[levels.LevelTwo]),
    ('Level 3',[levels.LevelThree]),
    ('Level 4',[levels.LevelFour]),
    ('Level 5',[levels.LevelFive]),
    ('Level 6',[levels.LevelSix]),
    ]

class SelectLevel(engine.State):
    background = os.path.join("data", "power_plant.jpg")
    sound_file = os.path.join("data", "menu.ogg")
    
    def init(self):
        self.next_state = None
        self.app = gui.App(theme=gui.Theme(["data/themes/default"]))
        self.image = pygame.transform.scale(pygame.image.load(self.background), 
                              (self.game.parameters.screenWidth, self.game.parameters.screenHeight))
        self.sound = pygame.mixer.Sound( self.sound_file )
        self.gui_init()
        self.sound.play(-1)
        
    def event(self, evt):
        return self.app.event(evt)
        
    def loop(self):
        if self.next_state:
            self.sound.stop()
            return self.next_state
            
    def paint(self, screen):
        screen.blit(self.image,(0,0))
        self.app.paint(screen)
        pygame.display.flip()
        
    def update(self, screen):
        self.paint(screen)

    def gui_init(self):
        global LevelList
        space = 10
        table = gui.Table()
        
        for levelName, levelData in LevelList:
            table.tr()
            leveln = gui.Button(levelName+": "+levelData[0].name)
            leveln.connect(gui.CLICK, self.onNewGame, levelData)
            table.td( leveln )
            table.tr(); table.td( gui.Spacer(0, space) )
            table.tr(); table.td( gui.Spacer(0, space) )
        
        table.tr()
        back = gui.Button("Back")
        back.connect(gui.CLICK, self.onBack, None)
        table.td( back )
        self.app.init(table)

    def onBack(self, *args):
        self.next_state = MainMenu(self.game)
        
    def onNewGame(self, choosedLevel, *args):
        level = dict(level=choosedLevel[0], next_state=MainMenu, score_state=ShowScore)
        ShowTutor.text = choosedLevel[0].description
        self.next_state = ShowTutor(self.game, LevelEditor, level)

class ShowTutor(textscroller.TextScroller):
    sound_file = os.path.join("data","sounds/Game Music.ogg")
    background = os.path.join("data", "board.jpg")
    color = (1,1,1)




def sendHighScore(scores, userName, machine, levelID):
    #for what in ['average', 'research', 'development', 'replication']:
    params = urllib.urlencode({'buildTime': scores['development'],
                                'executionTime': scores['average'],
                                'userName': userName,
                                'machine': machine,
                                'replicationCost': scores['replication'],
                                'researchCost': scores['research'],
                                'levelID': levelID})
    request = urllib.urlopen('http://stim.game-host.org:8080/addHighScore', 
                params)
if 0:
    sendHighScore(111111, 222222, "SirNubis", "{34:'aoeu',56:00}", 878, 999, 2)

class ShowScore(engine.State):
    background = os.path.join("data", "YES.jpg")
    sound_file = os.path.join("data", "menu.ogg")
    def init(self, *args, **kwargs):
        self.next_state = None
        self.app = gui.App(theme=gui.Theme(["data/themes/default"]))
        self.image = pygame.transform.scale(pygame.image.load(self.background), 
                              (self.game.parameters.screenWidth, self.game.parameters.screenHeight))
        self.sound = pygame.mixer.Sound( self.sound_file )
        self.gui_init()
        self.sound.play(-1)
       
    def event(self, evt):
        return self.app.event(evt)

        
    def loop(self):
        if self.next_state:
            self.sound.stop()
            return self.next_state
            
    def paint(self, screen):
        screen.blit(self.image,(0,0))
        self.app.paint(screen)
        pygame.display.flip()
        
    def update(self, screen):
        self.paint(screen)

    def gui_init(self):
        space = 10
        table = gui.Table()
        
        ScoreDesc = {
            'average': 'Machine average runtime',
            'research':'Research cost',
            'development':'Development time',
            'replication':'Replication cost',
             }
            
        for what in ['average', 'research', 'development', 'replication']:
            table.tr()
            label = gui.Label(ScoreDesc[what])
            table.td( label )
            value = gui.Label(str(self.value['scores'][what]))
            table.td( value ); 
            table.tr(); table.td( gui.Spacer(0, space) )
            table.tr(); table.td( gui.Spacer(0, space) )
        
        table.tr()
        back = gui.Button("Back to Main")
        back.connect(gui.CLICK, self.onBack, None)
        table.td( back )
        
        submit = gui.Button("Submit score")
        submit.connect(gui.CLICK, self.onSubmit, None)
        table.td( submit )
        self.app.init(table)

    def onBack(self, *args):
        self.next_state = MainMenu(self.game)
        
    def onSubmit(self, *args):
        self.next_state = SubmitScore(self.game, self.value )
        

class SubmitScore(engine.State):
    background = os.path.join("data", "YES.jpg")
    sound_file = os.path.join("data", "menu.ogg")
    def init(self, *args, **kwargs):
        self.next_state = None
        self.app = gui.App(theme=gui.Theme(["data/themes/default"]))
        self.image = pygame.transform.scale(pygame.image.load(self.background), 
                              (self.game.parameters.screenWidth, self.game.parameters.screenHeight))
        self.sound = pygame.mixer.Sound( self.sound_file )
        self.scores = self.value['scores']
        self.level = self.value['level']
        self.gui_init()
        self.sound.play(-1)
       
    def event(self, evt):
        return self.app.event(evt)
        
    def loop(self):
        if self.next_state:
            self.sound.stop()
            return self.next_state
            
    def paint(self, screen):
        screen.blit(self.image,(0,0))
        self.app.paint(screen)
        pygame.display.flip()
        
    def update(self, screen):
        self.paint(screen)

    def gui_init(self):
        space = 10
        table = gui.Table()
        
        ScoreDesc = {
            'average': 'Machine average runtime',
            'research':'Research cost',
            'development':'Development time',
            'replication':'Replication cost',
             }

        #f = gui.Form()
        table.tr()
        label = gui.Label('Name')
        self.value = gui.Form()
        table.td(gui.Label("Open: "))
        self.nameui = gui.Input(name="fname")
        table.td(self.nameui, colspan=3)

        table.tr(); table.td( gui.Spacer(0, space) )
        table.tr(); table.td( gui.Spacer(0, space) )
            
        for what in ['average', 'research', 'development', 'replication']:
            table.tr()
            label = gui.Label(ScoreDesc[what])
            table.td( label )
            value = gui.Label(str(self.scores[what]))
            table.td( value ); 
            table.tr(); table.td( gui.Spacer(0, space) )
            table.tr(); table.td( gui.Spacer(0, space) )
        
        table.tr()
        back = gui.Button("Back to Main")
        back.connect(gui.CLICK, self.onBack, None)
        table.td( back )
        
        submit = gui.Button("Submit score")
        submit.connect(gui.CLICK, self.onSubmit, None)
        table.td( submit )
        self.app.init(table)

    def onBack(self, *args):
        self.next_state = MainMenu(self.game)
        
    def onSubmit(self, *args):
        #print self.level
        sendHighScore(self.scores, self.nameui.value, '<machine/>', self.level.levelno)
        self.next_state = MainMenu(self.game)
        

class ViewHiscores(engine.State):
    background = os.path.join("data", "sunset.jpg")
    def __init__(self, game, next_state=MainMenu):
        engine.State.__init__(self, game)
        webbrowser.open(game_config.highScoresURL)
        self.quit = 1

    def loop(self):
        if self.quit:
            return MainMenu(self.game)



        
class StimGame(engine.Game):
    def init(self):
        self.parameters = game_config
        self.screen = pygame.display.set_mode(
                          (self.parameters.screenWidth, self.parameters.screenHeight), 
                          SWSURFACE | self.parameters.fullscreen)
        self.screen.fill((255,255,255))
        pygame.display.flip()
        pygame.mixer.init()
        pygame.font.init()
        self.timer = timer.Timer(self.parameters.framesPerSecond)
        

    def tick(self):
        self.timer.tick()
        
        
        
if __name__ == "__main__":
    game = StimGame()
    game.run(PlaySlideShow(game, dict(slideshow="intro.py", next_state=MainMenu)))
