from pgu import engine, gui, timer
import pygame
from pygame.locals import *
import os

class Transition:
    def __init__(self, duration, *args):
        self.duration = duration
        self.init(duration, *args)
            
    def getSurface(self, time):
        raise NotImplementedError()
    


class FromImage(Transition):
    def init(self, duration, image_start_file, image_end_file):
        self.start_image = pygame.image.load(image_start_file).convert()
        self.end_image = pygame.image.load(image_end_file).convert()        
        self.size = (self.start_image.get_width(), self.start_image.get_height())
        
    def getSurface(self, time):
        alpha = time*255/self.duration 
        surface = pygame.surface.Surface( self.size )
        self.start_image.set_alpha( 255-alpha )
        self.end_image.set_alpha( alpha )
        surface.blit( self.start_image, (0,0) )
        surface.blit( self.end_image, (0,0) )
        return surface

def FromColor(duration, color, image_end):
    t = FromImage(duration, image_end, image_end)
    t.start_image = pygame.surface.Surface(t.size)
    t.start_image.fill( color )
    return t

def ToColor(duration, image_start, color):
    t = FromImage(duration, image_start, image_start)
    t.end_image = pygame.surface.Surface(t.size)
    t.end_image.fill( color )
    return t

 
class ShowImage(Transition):
    def init(self, duration, img_source):
        self.image = pygame.image.load(img_source).convert()
    def getSurface(self, time):
        return self.image
    

class SlideShow:
    slides = []
    
    def __init__(self):
        def offset(seq):
            off = 0
            out_seq = []
            for e in seq:
                off += e
                out_seq.append( off )
            return out_seq
        
        starts = offset(([0]+[t.duration for t in self.slides])[:-1])
        ends = offset([t.duration for t in self.slides])
        self.indexes = zip(starts, ends)
        
        
    def slideAndOffset(self, time):
        pos = -1
        off = -1
        for i, (start, end) in enumerate(self.indexes):
            if time < end and time >= start:
                pos = i
                off = time-start
                break
        return pos, off
    
    def getSurface(self, time):
        pos, off = self.slideAndOffset(time)
        if pos < 0:
            return None
        return self.slides[pos].getSurface( off )
        
